@@ -64,6 +64,34 @@ class API(api.API):
         :returns: :class:`Command` with RowView result
         """
 
+    @abc.abstractmethod
+    def ls_set_dns_records(self, switch_uuid, dns_uuids):
+        """Sets 'dns_records' column on the switch with uuid 'switch_uuid'
+
+        :returns: :class:`Command` with RowView result
+        """
+
+    @abc.abstractmethod
+    def ls_clear_dns_records(self, switch):
+        """Clears 'dns_records' from the switch with uuid 'switch_uuid'
+
+        :returns: :class:`Command` with RowView result
+        """
+
+    @abc.abstractmethod
+    def ls_add_dns_record(self, switch_uuid, dns_uuid):
+        """Add the 'dns_record' to the switch's 'dns_records' list
+
+        :returns: :class:`Command` with RowView result
+        """
+
+    @abc.abstractmethod
+    def ls_remove_dns_record(self, switch_uuid, dns_uuid):
+        """Remove the 'dns_record' from the switch's 'dns_records' list
+
+        :returns: :class:`Command` with RowView result
+        """
+
     @abc.abstractmethod
     def acl_add(self, switch, direction, priority, match, action, log=False):
         """Add an ACL to 'switch'
@@ -335,6 +363,13 @@ class API(api.API):
         :returns:      :class:`Command` with RowView list result
         """
 
+    @abc.abstractmethod
+    def lr_get(self, router):
+        """Get logical router for 'router'
+
+        :returns: :class:`Command` with RowView result
+        """
+
     @abc.abstractmethod
     def lrp_add(self, router, port, mac, networks, peer=None, may_exist=False,
                 **columns):
@@ -657,3 +692,163 @@ class API(api.API):
         :type uuid:   string or uuid.UUID
         :returns:     :class:`Command` with dict result
         """
+
+    @abc.abstractmethod
+    def dns_add(self, **columns):
+        """Create a DNS row with columns
+
+        :param **columns:  Additional columns to directly set on the dns
+        :returns:          :class:`Command` with RowView result
+        """
+
+    @abc.abstractmethod
+    def dns_del(self, uuid):
+        """Delete DNS row with 'uuid'
+
+        :param uuid: The uuid of the DNS row to delete
+        :type uuid:  string or uuid.UUID
+        :returns:    :class:`Command` with no result
+        """
+
+    @abc.abstractmethod
+    def dns_get(self, uuid):
+        """Get DNS row with 'uuid'
+
+        :returns: :class:`Command` with RowView result
+        """
+
+    @abc.abstractmethod
+    def dns_list(self):
+        """Get all DNS rows
+
+        :returns: :class:`Command with RowView list result
+        """
+
+    @abc.abstractmethod
+    def dns_set_records(self, uuid, **records):
+        """Sets the 'records' field of the DNS row
+
+        :param uuid: The uuid of the DNS row to set the records with
+        :type uuid:  string or uuid.UUID
+        :param records: keys and values for the DNS 'records' dict
+        :type records:  key: string, value: string
+        :returns:    :class:`Command` with no result
+        """
+
+    @abc.abstractmethod
+    def dns_add_record(self, uuid, hostname, ips):
+        """Add the record 'hostname: ips' into the records column of the DNS
+
+        :param uuid: The uuid of the DNS row to add the record
+        :type uuid:  string or uuid.UUID
+        :param hostname: hostname as the key to the record dict
+        :type ips:  IPs as the value to the hostname key in the 'records'
+        :returns:    :class:`Command` with no result
+        """
+
+    @abc.abstractmethod
+    def dns_remove_record(self, uuid, hostname):
+        """Remove the 'hostname' from the 'records' field of the DNS row
+
+        :param uuid: The uuid of the DNS row to set the records with
+        :type uuid:  string or uuid.UUID
+        :param hostname: hostname as the key to the record dict
+        :returns:    :class:`Command` with no result
+        """
+
+    @abc.abstractmethod
+    def dns_set_external_ids(self, uuid, **external_ids):
+        """Sets the 'external_ids' field of the DNS row
+
+        :param uuid: The uuid of the DNS row to set the external_ids with
+        :type uuid:  string or uuid.UUID
+        :param external_ids: keys and values for the DNS 'external_ids' dict
+        :type external_ids:  key: string, value: string
+        :returns:    :class:`Command` with no result
+        """
+
+    @abc.abstractmethod
+    def pg_add(self, name, may_exist=False, **columns):
+        """Create a port group
+
+        :param name:        The name of the port group
+        :type name:         string
+        :param may_exist:   If True, don't fail if the port group already
+                            exists
+        :type may_exist:    bool
+        :param columns:     Additional columns to directly set on the port
+                            group (e.g external_ids, ports, acls)
+        :type columns:      dictionary
+        :returns:           :class:`Command` with RowView result
+        """
+
+    @abc.abstractmethod
+    def pg_del(self, name, if_exists=False):
+        """Delete a port group
+
+        :param name:        The name of the port group
+        :type name:         string
+        :param if_exists:   If True, don't fail if the router doesn't exist
+        :type if_exists:    boolean
+        :returns:           :class:`Command` with no result
+        """
+
+    @abc.abstractmethod
+    def pg_add_ports(self, pg_id, lsp):
+        """Add a list of logical port to a port group
+
+        :param pg_id:     The name or uuid of the port group
+        :type pg_id:      string or uuid.UUID
+        :param lsp:       A list of :class:`Command` with a Logical_Switch_Port
+                          instance result or UUID
+        :type lsp:        A list of :class:`Command` with a Logical_Switch_Port
+                          or string or uuid.UUID
+        A Logical_Switch_Port instance or string
+                          or uuid.UUID
+        :returns:         :class:`Command` with no result
+        """
+
+    @abc.abstractmethod
+    def pg_del_ports(self, pg_id, lsp, if_exists=False):
+        """Delete a list of logical port from a port group
+
+        :param pg_id:     The name or uuid of the port group
+        :type pg_id:      string or uuid.UUID
+        :param lsp:       A list of :class:`Command` with a Logical_Switch_Port
+                          instance result or UUID
+        :type lsp:        A list of :class:`Command` with a Logical_Switch_Port
+                          or string or uuid.UUID
+        :type if_exists:  If True, don't fail if the logical port(s) doesn't
+                          exist
+        :type if_exists:  boolean
+        :returns:         :class:`Command` with no result
+        """
+
+    @abc.abstractmethod
+    def pg_add_acls(self, pg_id, acl):
+        """Add a list of ACL to a port group
+
+        :param pg_id:     The name or uuid of the port group
+        :type pg_id:      string or uuid.UUID
+        :param acl:       The ACL instance or UUID
+        :type acl:        A list of ACL instance or string or uuid.UUID
+        :param acl:       A list of :class:`Command` with an ACL instance
+                          result or UUID
+        :type acl:        A :class:`Command` with an ACL or string
+                          r uuid.UUID
+        :returns:         :class:`Command` with no result
+        """
+
+    @abc.abstractmethod
+    def pg_del_acls(self, pg_id, acl, if_exists=False):
+        """Delete a list of ACL from a port group
+
+        :param pg_id:     The name or uuid of the port group
+        :type pg_id:      string or uuid.UUID
+        :type acl:        A list of ACL instance or string or uuid.UUID
+        :param acl:       A list of :class:`Command` with an ACL instance
+                          result or UUID
+        :type if_exists:  If True, don't fail if the ACL(s) doesn't exist
+        :type if_exists:  boolean
+        :returns:         :class:`Command` with no result
+        """
