@@ -12,6 +12,7 @@
 import re
 
 import netaddr
+from oslo_utils import uuidutils
 
 from ovsdbapp.backend.ovs_idl import command as cmd
 from ovsdbapp.backend.ovs_idl import idlutils
@@ -80,29 +81,26 @@ class LsGetCommand(cmd.BaseGetRowCommand):
     table = 'Logical_Switch'
 
 
-class _AclAddHelper(cmd.AddCommand):
+class AclAddCommand(cmd.AddCommand):
     table_name = 'ACL'
 
-    def __init__(self, api, entity, direction, priority, match, action,
-                 log=False, may_exist=False, severity=None, name=None,
-                 **external_ids):
+    def __init__(self, api, switch, direction, priority, match, action,
+                 log=False, may_exist=False, **external_ids):
         if direction not in ('from-lport', 'to-lport'):
             raise TypeError("direction must be either from-lport or to-lport")
         if not 0 <= priority <= const.ACL_PRIORITY_MAX:
-            raise ValueError("priority must be beween 0 and %s, inclusive" % (
+            raise ValueError("priority must be between 0 and %s, inclusive" % (
                              const.ACL_PRIORITY_MAX))
         if action not in ('allow', 'allow-related', 'drop', 'reject'):
             raise TypeError("action must be allow/allow-related/drop/reject")
-        super(_AclAddHelper, self).__init__(api)
-        self.entity = entity
+        super(AclAddCommand, self).__init__(api)
+        self.switch = switch
         self.direction = direction
         self.priority = priority
         self.match = match
         self.action = action
         self.log = log
         self.may_exist = may_exist
-        self.severity = severity
-        self.name = name
         self.external_ids = external_ids
 
     def acl_match(self, row):
@@ -111,8 +109,8 @@ class _AclAddHelper(cmd.AddCommand):
                 self.match == row.match)
 
     def run_idl(self, txn):
-        entity = self.api.lookup(self.lookup_table, self.entity)
-        acls = [acl for acl in entity.acls if self.acl_match(acl)]
+        ls = self.api.lookup('Logical_Switch', self.switch)
+        acls = [acl for acl in ls.acls if self.acl_match(acl)]
         if acls:
             if self.may_exist:
                 self.result = rowview.RowView(acls[0])
@@ -125,87 +123,45 @@ class _AclAddHelper(cmd.AddCommand):
         acl.match = self.match
         acl.action = self.action
         acl.log = self.log
-        acl.severity = self.severity
-        acl.name = self.name
-        entity.addvalue('acls', acl)
+        ls.addvalue('acls', acl)
         for col, value in self.external_ids.items():
             acl.setkey('external_ids', col, value)
         self.result = acl.uuid
 
 
-class AclAddCommand(_AclAddHelper):
-    lookup_table = 'Logical_Switch'
-
-    def __init__(self, api, switch, direction, priority, match, action,
-                 log=False, may_exist=False, severity=None, name=None,
-                 **external_ids):
-        # NOTE: we're overriding the constructor here to not break any
-        # existing callers before we introduced Port Groups.
-        super(AclAddCommand, self).__init__(api, switch, direction, priority,
-                                            match, action, log, may_exist,
-                                            severity, name, **external_ids)
-
-
-class PgAclAddCommand(_AclAddHelper):
-    lookup_table = 'Port_Group'
-
-
-class _AclDelHelper(cmd.BaseCommand):
-    def __init__(self, api, entity, direction=None,
+class AclDelCommand(cmd.BaseCommand):
+    def __init__(self, api, switch, direction=None,
                  priority=None, match=None):
         if (priority is None) != (match is None):
             raise TypeError("Must specify priority and match together")
         if priority is not None and not direction:
             raise TypeError("Cannot specify priority/match without direction")
-        super(_AclDelHelper, self).__init__(api)
-        self.entity = entity
+        super(AclDelCommand, self).__init__(api)
+        self.switch = switch
         self.conditions = []
         if direction:
             self.conditions.append(('direction', '=', direction))
             # priority can be 0
-            if match:  # and therefore priority due to the above check
+            if match:  # and therefor prioroity due to the above check
                 self.conditions += [('priority', '=', priority),
                                     ('match', '=', match)]
 
     def run_idl(self, txn):
-        entity = self.api.lookup(self.lookup_table, self.entity)
-        for acl in [a for a in entity.acls
+        ls = self.api.lookup('Logical_Switch', self.switch)
+        for acl in [a for a in ls.acls
                     if idlutils.row_match(a, self.conditions)]:
-            entity.delvalue('acls', acl)
+            ls.delvalue('acls', acl)
             acl.delete()
 
 
-class AclDelCommand(_AclDelHelper):
-    lookup_table = 'Logical_Switch'
-
-    def __init__(self, api, switch, direction=None,
-                 priority=None, match=None):
-        # NOTE: we're overriding the constructor here to not break any
-        # existing callers before we introduced Port Groups.
-        super(AclDelCommand, self).__init__(api, switch, direction, priority,
-                                            match)
-
-
-class PgAclDelCommand(_AclDelHelper):
-    lookup_table = 'Port_Group'
-
-
-class _AclListHelper(cmd.BaseCommand):
-    def __init__(self, api, entity):
-        super(_AclListHelper, self).__init__(api)
-        self.entity = entity
+class AclListCommand(cmd.BaseCommand):
+    def __init__(self, api, switch):
+        super(AclListCommand, self).__init__(api)
+        self.switch = switch
 
     def run_idl(self, txn):
-        entity = self.api.lookup(self.lookup_table, self.entity)
-        self.result = [rowview.RowView(acl) for acl in entity.acls]
-
-
-class AclListCommand(_AclListHelper):
-    lookup_table = 'Logical_Switch'
-
-
-class PgAclListCommand(_AclListHelper):
-    lookup_table = 'Port_Group'
+        ls = self.api.lookup('Logical_Switch', self.switch)
+        self.result = [rowview.RowView(acl) for acl in ls.acls]
 
 
 class LspAddCommand(cmd.AddCommand):
@@ -391,7 +347,7 @@ class LspGetPortSecurityCommand(cmd.BaseCommand):
         self.result = lsp.port_security
 
 
-class LspGetUpCommand(cmd.ReadOnlyCommand):
+class LspGetUpCommand(cmd.BaseCommand):
     def __init__(self, api, port):
         super(LspGetUpCommand, self).__init__(api)
         self.port = port
@@ -413,7 +369,7 @@ class LspSetEnabledCommand(cmd.BaseCommand):
         lsp.enabled = self.is_enabled
 
 
-class LspGetEnabledCommand(cmd.ReadOnlyCommand):
+class LspGetEnabledCommand(cmd.BaseCommand):
     def __init__(self, api, port):
         super(LspGetEnabledCommand, self).__init__(api)
         self.port = port
@@ -435,7 +391,7 @@ class LspSetTypeCommand(cmd.BaseCommand):
         lsp.type = self.port_type
 
 
-class LspGetTypeCommand(cmd.ReadOnlyCommand):
+class LspGetTypeCommand(cmd.BaseCommand):
     def __init__(self, api, port):
         super(LspGetTypeCommand, self).__init__(api)
         self.port = port
@@ -456,7 +412,7 @@ class LspSetOptionsCommand(cmd.BaseCommand):
         lsp.options = self.options
 
 
-class LspGetOptionsCommand(cmd.ReadOnlyCommand):
+class LspGetOptionsCommand(cmd.BaseCommand):
     def __init__(self, api, port):
         super(LspGetOptionsCommand, self).__init__(api)
         self.port = port
@@ -477,7 +433,7 @@ class LspSetDhcpV4OptionsCommand(cmd.BaseCommand):
         lsp.dhcpv4_options = self.dhcpopt_uuid
 
 
-class LspGetDhcpV4OptionsCommand(cmd.ReadOnlyCommand):
+class LspGetDhcpV4OptionsCommand(cmd.BaseCommand):
     def __init__(self, api, port):
         super(LspGetDhcpV4OptionsCommand, self).__init__(api)
         self.port = port
@@ -514,7 +470,7 @@ class DhcpOptionsDelCommand(cmd.BaseCommand):
         dhcpopt.delete()
 
 
-class DhcpOptionsListCommand(cmd.ReadOnlyCommand):
+class DhcpOptionsListCommand(cmd.BaseCommand):
     def run_idl(self, txn):
         self.result = [rowview.RowView(r) for
                        r in self.api.tables['DHCP_Options'].rows.values()]
@@ -535,7 +491,7 @@ class DhcpOptionsSetOptionsCommand(cmd.BaseCommand):
         dhcpopt.options = self.options
 
 
-class DhcpOptionsGetOptionsCommand(cmd.ReadOnlyCommand):
+class DhcpOptionsGetOptionsCommand(cmd.BaseCommand):
     def __init__(self, api, dhcpopt_uuid):
         super(DhcpOptionsGetOptionsCommand, self).__init__(api)
         self.dhcpopt_uuid = dhcpopt_uuid
@@ -590,12 +546,16 @@ class LrDelCommand(cmd.BaseCommand):
             raise RuntimeError(msg)
 
 
-class LrListCommand(cmd.ReadOnlyCommand):
+class LrListCommand(cmd.BaseCommand):
     def run_idl(self, txn):
         self.result = [rowview.RowView(r) for
                        r in self.api.tables['Logical_Router'].rows.values()]
 
 
+class LrGetCommand(cmd.BaseGetRowCommand):
+    table = 'Logical_Router'
+
+
 class LrpAddCommand(cmd.BaseCommand):
     def __init__(self, api, router, port, mac, networks,
                  peer=None, may_exist=False, **columns):
@@ -656,7 +616,7 @@ class LrpDelCommand(PortDelCommand):
             if_exists)
 
 
-class LrpListCommand(cmd.ReadOnlyCommand):
+class LrpListCommand(cmd.BaseCommand):
     def __init__(self, api, router):
         super(LrpListCommand, self).__init__(api)
         self.router = router
@@ -677,7 +637,7 @@ class LrpSetEnabledCommand(cmd.BaseCommand):
         lrp.enabled = self.is_enabled
 
 
-class LrpGetEnabledCommand(cmd.ReadOnlyCommand):
+class LrpGetEnabledCommand(cmd.BaseCommand):
     def __init__(self, api, port):
         super(LrpGetEnabledCommand, self).__init__(api)
         self.port = port
@@ -758,7 +718,7 @@ class LrRouteDelCommand(cmd.BaseCommand):
             raise RuntimeError(msg)
 
 
-class LrRouteListCommand(cmd.ReadOnlyCommand):
+class LrRouteListCommand(cmd.BaseCommand):
     def __init__(self, api, router):
         super(LrRouteListCommand, self).__init__(api)
         self.router = router
@@ -867,7 +827,7 @@ class LrNatDelCommand(cmd.BaseCommand):
                                        match=self.match_ip)
 
 
-class LrNatListCommand(cmd.ReadOnlyCommand):
+class LrNatListCommand(cmd.BaseCommand):
     def __init__(self, api, router):
         super(LrNatListCommand, self).__init__(api)
         self.router = router
@@ -934,7 +894,7 @@ class LbDelCommand(cmd.BaseCommand):
                 raise
 
 
-class LbListCommand(cmd.ReadOnlyCommand):
+class LbListCommand(cmd.BaseCommand):
     def run_idl(self, txn):
         self.result = [rowview.RowView(r)
                        for r in self.api.tables['Load_Balancer'].rows.values()]
@@ -979,7 +939,7 @@ class LrLbDelCommand(cmd.BaseCommand):
             raise
 
 
-class LrLbListCommand(cmd.ReadOnlyCommand):
+class LrLbListCommand(cmd.BaseCommand):
     def __init__(self, api, router):
         super(LrLbListCommand, self).__init__(api)
         self.router = router
@@ -1028,7 +988,7 @@ class LsLbDelCommand(cmd.BaseCommand):
             raise
 
 
-class LsLbListCommand(cmd.ReadOnlyCommand):
+class LsLbListCommand(cmd.BaseCommand):
     def __init__(self, api, switch):
         super(LsLbListCommand, self).__init__(api)
         self.switch = switch
@@ -1063,7 +1023,7 @@ class DnsGetCommand(cmd.BaseGetRowCommand):
     table = 'DNS'
 
 
-class DnsListCommand(cmd.ReadOnlyCommand):
+class DnsListCommand(cmd.BaseCommand):
     def run_idl(self, txn):
         table = self.api.tables['DNS']
         self.result = [rowview.RowView(r) for r in table.rows.values()]
@@ -1113,12 +1073,11 @@ class PgAddCommand(cmd.AddCommand):
             try:
                 pg = self.api.lookup(self.table_name, self.name)
                 self.result = rowview.RowView(pg)
-                return
             except idlutils.RowNotFound:
                 pass
 
         pg = txn.insert(self.api._tables[self.table_name])
-        pg.name = self.name or ""
+        pg.name = self.name
         self.set_columns(pg, **self.columns)
         self.result = pg.uuid
 
@@ -1141,34 +1100,40 @@ class PgDelCommand(cmd.BaseCommand):
             raise RuntimeError('Port group %s does not exist' % self.name)
 
 
-class _PgUpdatePortsHelper(cmd.BaseCommand):
+class _PgUpdateHelper(cmd.BaseCommand):
     method = None
 
-    def __init__(self, api, port_group, lsp=None, if_exists=False):
-        super(_PgUpdatePortsHelper, self).__init__(api)
+    def __init__(self, api, port_group, lsp=None, acl=None, if_exists=False):
+        super(_PgUpdateHelper, self).__init__(api)
         self.port_group = port_group
         self.lsp = [] if lsp is None else self._listify(lsp)
+        self.acl = [] if acl is None else self._listify(acl)
         self.if_exists = if_exists
 
     def _listify(self, res):
         return res if isinstance(res, (list, tuple)) else [res]
 
-    def _run_method(self, pg, port):
-        if not port:
+    def _fetch_resource(self, type_, uuid_):
+        table = 'Logical_Switch_Port' if type_ == 'ports' else 'ACL'
+        return self.api.lookup(table, uuid_)
+
+    def _run_method(self, pg, column, resource):
+        if not resource:
             return
 
-        if isinstance(port, cmd.BaseCommand):
-            port = port.result
-        elif utils.is_uuid_like(port):
+        if isinstance(resource, cmd.BaseCommand):
+            resource = resource.result
+        elif uuidutils.is_uuid_like(resource):
             try:
-                port = self.api.lookup('Logical_Switch_Port', port)
+                resource = self._fetch_resource(column, resource)
             except idlutils.RowNotFound:
                 if self.if_exists:
                     return
                 raise RuntimeError(
-                    'Port %s does not exist' % port)
+                    'Resource %(res)s of type "%(type)s" does not exist' %
+                    {'res': resource, 'type': column})
 
-        getattr(pg, self.method)('ports', port)
+        getattr(pg, self.method)(column, resource)
 
     def run_idl(self, txn):
         try:
@@ -1178,16 +1143,15 @@ class _PgUpdatePortsHelper(cmd.BaseCommand):
                                self.port_group)
 
         for lsp in self.lsp:
-            self._run_method(pg, lsp)
+            self._run_method(pg, 'ports', lsp)
+
+        for acl in self.acl:
+            self._run_method(pg, 'acls', acl)
 
 
-class PgAddPortCommand(_PgUpdatePortsHelper):
+class PgAddDataCommand(_PgUpdateHelper):
     method = 'addvalue'
 
 
-class PgDelPortCommand(_PgUpdatePortsHelper):
+class PgDelDataCommand(_PgUpdateHelper):
     method = 'delvalue'
-
-
-class PgGetCommand(cmd.BaseGetRowCommand):
-    table = 'Port_Group'
